#!/usr/bin/python
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import sys
import dbus
import time
import os
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GLib,Gio,Gdk,GObject
from dbus.mainloop.glib import DBusGMainLoop
from grub_lib import *
from basic_lib import *

class Advanced_Option_UI(object):
    def __init__(self, uidir, boot_manager, grub_cfg_dir="/etc/default/grub.d/", grub_cfg_file="/etc/default/grub.d/z_boot_mgr_grub.cfg"):
        self.grub_cfg_dir = grub_cfg_dir
        self.grub_cfg_file = grub_cfg_file
        import gettext as gettext2
        gettext2.bindtextdomain('boot_manager', "/usr/share/locale")
        gettext2.textdomain('boot_manager')
        self._ = gettext2.gettext
        provider = Gtk.CssProvider()
        provider.load_from_data(b'''\
            #advanced_options_window {
                background: #ffffff;
                border: none;
                border: #cccccc 1px solid;
            }
            #cpu_checkbutton,
            #mem_checkbutton,
            #bootlog_checkbutton,
            #splash_checkbutton,
            #more_boot_log_checkbutton,
            #debug_checkbutton,
            #debug_port_checkbutton,
            #baud_rate_checkbutton {
                color: #333333;
                font-size: 12px;
            }
            #cpu_checkbutton:insensitive,
            #mem_checkbutton:insensitive,
            #bootlog_checkbutton:insensitive,
            #splash_checkbutton:insensitive,
            #more_boot_log_checkbutton:insensitive,
            #debug_checkbutton:insensitive,
            #debug_port_checkbutton:insensitive,
            #baud_rate_checkbutton:insensitive {
                color: #999999;
                font-size: 12px;
            }
            #advanced_warning_label {
                color: #999999;
                font-size: 12px;
            }
            #reset_button {
                font-size: 16px;
                background: #44a6e5;
                color: #ffffff;
            }
            #reset_button:hover {
                background: #61b2e9;
            }
            #reset_button:active {
                background: #3c95cf;
            }
            #ok_button,
            #cancel_button {
                font-size: 16px;
                background: #dddddd;
                color: #333333;
            }
            #ok_button:hover,
            #cancel_button:hover {
                background: #e2e2e2;
            }
            #ok_button:active,
            #cancel_button:active {
                background: #c7c7c7;
            }
            ''')
        Gtk.StyleContext.add_provider_for_screen(
            Gdk.Screen.get_default(),
            provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

        self.boot_mgr = boot_manager
        self.builder = Gtk.Builder()
        self.builder.add_from_file(uidir + "/advanced_options.ui")
        self.builder.connect_signals(self)
        self.window = self.builder.get_object("advanced_options_window")
        self.window.set_title(self._("Advanced Options"))
        self.window.set_name("advanced_options_window")
        self.window.connect("delete-event", self.close)
        self.window.set_position(Gtk.WindowPosition.CENTER)

        # button
        self.cpu_checkbutton = self.builder.get_object("cpu_checkbutton")
        self.cpu_checkbutton.set_name("cpu_checkbutton")
        self.cpu_checkbutton.set_label(self._("Number of processors:"))
        self.cpu_comboboxtext = self.builder.get_object("cpu_comboboxtext")
        self.cpu_comboboxtext.set_name("cpu_comboboxtext")

        self.mem_checkbutton = self.builder.get_object("mem_checkbutton")
        self.mem_checkbutton.set_name("mem_checkbutton")
        self.mem_checkbutton.set_label(self._("Max memory map address:") + "(MB)")
        self.mem_spinbutton = self.builder.get_object("mem_spinbutton")
        self.mem_spinbutton.set_name("mem_spinbutton")

        self.bootlog_checkbutton = self.builder.get_object("bootlog_checkbutton")
        self.bootlog_checkbutton.set_name("bootlog_checkbutton")
        self.bootlog_checkbutton.set_label(self._("Boot log"))

        self.splash_checkbutton = self.builder.get_object("splash_checkbutton")
        self.splash_checkbutton.set_name("splash_checkbutton")
        self.splash_checkbutton.set_label(self._("No GUI boot"))

        self.more_boot_log_checkbutton = self.builder.get_object("more_boot_log_checkbutton")
        self.more_boot_log_checkbutton.set_name("more_boot_log_checkbutton")
        self.more_boot_log_checkbutton.set_label(self._("More Boot log"))

        self.debug_checkbutton = self.builder.get_object("debug_checkbutton")
        self.debug_checkbutton.set_name("debug_checkbutton")
        self.debug_checkbutton.set_label(self._("Debug"))

        self.debug_port_checkbutton = self.builder.get_object("debug_port_checkbutton")
        self.debug_port_checkbutton.set_name("debug_port_checkbutton")
        self.debug_port_checkbutton.set_label(self._("Debug port:"))
        self.debug_port_comboboxtext = self.builder.get_object("debug_port_comboboxtext")
        self.debug_port_comboboxtext.set_name("debug_port_comboboxtext")

        self.baud_rate_checkbutton = self.builder.get_object("baud_rate_checkbutton")
        self.baud_rate_checkbutton.set_name("baud_rate_checkbutton")
        self.baud_rate_checkbutton.set_label(self._("Baud rate:"))
        self.baud_rate_comboboxtext = self.builder.get_object("baud_rate_comboboxtext")
        self.baud_rate_comboboxtext.set_name("baud_rate_comboboxtext")

        self.kernel_version_label = self.builder.get_object("kernel_version_label")
        self.kernel_version_label.set_name("kernel_version_label")
        self.kernel_version_label.set_markup(self._("Kernel Version:"))
        self.kernel_scrolledwindow = self.builder.get_object("kernel_scrolledwindow")
        self.kernel_scrolledwindow.set_name("kernel_scrolledwindow")
        self.kernel_viewport = self.builder.get_object("kernel_viewport")
        self.kernel_viewport.set_name("kernel_viewport")
        self.kernel_box = self.builder.get_object("kernel_box")
        self.kernel_box.set_name("kernel_box")

        self.reset_button = self.builder.get_object("reset_button")
        self.reset_button.set_name("reset_button")
        self.reset_button.set_label(self._("Reset"))

        self.ok_button = self.builder.get_object("ok_button")
        self.ok_button.set_name("ok_button")
        self.ok_button.set_label(self._("OK"))

        self.advanced_warning_label = self.builder.get_object("advanced_warning_label")
        self.advanced_warning_label.set_name("advanced_warning_label")
        self.advanced_warning_label.set_max_width_chars(10)
        self.advanced_warning_label.set_label(self._("Tips: Some functions is limit by UEFI,so that you may can't use some of them."))

        self.cancel_button = self.builder.get_object("cancel_button")
        self.cancel_button.set_name("cancel_button")
        self.cancel_button.set_label(self._("Cancel"))

        self.grub_env = get_grub_env_from_file()
        self.grub_env_config = get_config_from_env( self.grub_env)
        self.init_default_ui(self.grub_env_config)
        self.destroy_flag = False

    def init_default_ui(self, grub_env_config):
        min_mem_capacity = 600
        # 引导日志
        if grub_env_config["quiet"] == False:
            self.bootlog_checkbutton.set_active(True)
        # 动画启动
        if grub_env_config["splash"] == False:
            self.splash_checkbutton.set_active(True)
        # 更多日志
        if grub_env_config["more_log"] == True:
            self.more_boot_log_checkbutton.set_active(True)
        # cpus
        self.cpu_comboboxtext.set_sensitive(False)
        get_cpu_nums_res = get_cpu_nums()
        if get_cpu_nums_res.getError():
            self.cpu_checkbutton.set_sensitive(False)
        else:
            for num in range(get_cpu_nums_res.getValue()):
                self.cpu_comboboxtext.append_text( str(num + 1))
            # 如果已经设置cpu
            if ( not grub_env_config["maxcpus"] == 0) and ( not grub_env_config["maxcpus"] > get_cpu_nums_res.getValue()):
                # self.cpu_comboboxtext.set_sensitive(True)
                self.cpu_checkbutton.set_active(True)
                self.cpu_comboboxtext.set_active(grub_env_config["maxcpus"] - 1)
        # mem
        self.mem_spinbutton.set_sensitive(False)
        get_mem_capacity_res = get_mem_capacity2()
        if get_mem_capacity_res.getError():
            self.mem_checkbutton.set_sensitive(False)
        else:
            mem_capacity = get_mem_capacity_res.getValue()
            # 设置内存范围
            adjustment = Gtk.Adjustment(0, min_mem_capacity, mem_capacity, 1, 100, 0)
            self.mem_spinbutton.set_adjustment(adjustment)
            # self.mem_spinbutton.set_range( 0, mem_capacity)
            if ( not grub_env_config["mem"] == 0)  and ( not grub_env_config["mem"] > mem_capacity):
                # self.mem_spinbutton.set_sensitive(True)
                self.mem_checkbutton.set_active(True)
                if grub_env_config["mem"] <= min_mem_capacity:
                    self.mem_spinbutton.set_value(min_mem_capacity)
                else:
                    self.mem_spinbutton.set_value(grub_env_config["mem"])
            else:
                self.mem_spinbutton.set_value(mem_capacity)
        # Debug
        self.debug_port_comboboxtext.append_text("ttyS0")
        self.debug_port_comboboxtext.append_text("ttyUSB0")
        self.baud_rate_comboboxtext.append_text("9600")
        self.baud_rate_comboboxtext.append_text("19200")
        self.baud_rate_comboboxtext.append_text("38400")
        self.baud_rate_comboboxtext.append_text("57600")
        self.baud_rate_comboboxtext.append_text("115200")

        self.debug_port_checkbutton.set_sensitive(False)
        self.debug_port_comboboxtext.set_sensitive(False)
        self.baud_rate_checkbutton.set_sensitive(False)
        self.baud_rate_comboboxtext.set_sensitive(False)
        # if debug
        if grub_env_config["debug"] == True:
            self.debug_checkbutton.set_active(True)
            # serial debug
            if grub_env_config["serial_debug"] == True:
                self.debug_port_checkbutton.set_active(True)
                self.debug_port_comboboxtext.set_active(0)
                baud_rate = grub_env_config["serial_baud_rate"]
            # usb debug
            elif grub_env_config["usb_debug"] == True:
                self.debug_port_checkbutton.set_active(True)
                self.debug_port_comboboxtext.set_active(1)
                baud_rate = grub_env_config["usb_baud_rate"]
            # set baud rate
            self.baud_rate_checkbutton.set_active(True)
            if baud_rate <= 9600:
                self.baud_rate_comboboxtext.set_active(0)
            elif baud_rate <= 19200:
                self.baud_rate_comboboxtext.set_active(1)
            elif baud_rate <= 38400:
                self.baud_rate_comboboxtext.set_active(2)
            elif baud_rate <= 57600:
                self.baud_rate_comboboxtext.set_active(3)
            # elif baud_rate <= 115200:
            #     self.baud_rate_comboboxtext.set_active(4)
            else:
                self.baud_rate_comboboxtext.set_active(4)
        else:
            self.debug_port_checkbutton.set_active(True)
            self.baud_rate_checkbutton.set_active(True)
            self.debug_port_comboboxtext.set_active(0)
            self.baud_rate_comboboxtext.set_active(0)

        # kernel info
        self.init_kernel_info()

    def init_kernel_info(self):
        self.default_kernel_mgr = None
        self.storage = Kernel_Storage()
        current_os_info = get_10_linux_detail_info_from_cfg()

        if "GRUB_DEFAULT" in self.grub_env:
            grub_default = self.grub_env["GRUB_DEFAULT"]
            if grub_default.startswith("\"") or grub_default.startswith("\'"):
                grub_default = grub_default[1:]
            if grub_default.endswith("\"") or grub_default.endswith("\'"):
                grub_default = grub_default[:-1]
            if grub_default.split(">")[-1] == "0":
                default_kernel_menu_id = current_os_info["menu_id"]
            else:
                default_kernel_menu_id = grub_default.split(">")[-1]
        else:
            default_kernel_menu_id = current_os_info["menu_id"]

        # 根据内核版本降序排序
        for kernel_info in sorted(current_os_info["kernel_id"], reverse=True, key=lambda x:x['kernel']):
            kernel_mgr = Kernel_Manager(self.kernel_box, self.storage, kernel_info)
            if default_kernel_menu_id == kernel_info["menu_id"]:
                kernel_mgr.set_active(True)
            if current_os_info["menu_id"] == kernel_info["menu_id"]:
                self.default_kernel_mgr = kernel_mgr
            self.kernel_box.pack_start(kernel_mgr, False, False, 0)

    def reset_ui(self, widget):
        self.cpu_checkbutton.set_active(False)
        self.mem_checkbutton.set_active(False)
        self.bootlog_checkbutton.set_active(False)
        self.splash_checkbutton.set_active(False)
        self.more_boot_log_checkbutton.set_active(False)
        if self.default_kernel_mgr is not None:
            self.default_kernel_mgr.set_active(True)

    def on_debug_port_checkbutton_toggled(self, widget):
        if self.debug_port_checkbutton.get_active() == True and self.debug_checkbutton.get_active() == True:
            self.debug_port_comboboxtext.set_sensitive(True)
        else:
            self.debug_port_comboboxtext.set_sensitive(False)

    def on_baud_rate_checkbutton_toggled(self, widget):
        if self.baud_rate_checkbutton.get_active() == True and self.debug_checkbutton.get_active() == True:
            self.baud_rate_comboboxtext.set_sensitive(True)
        else:
            self.baud_rate_comboboxtext.set_sensitive(False)

    def on_more_boot_log_checkbutton_toggled(self, widget):
        if self.more_boot_log_checkbutton.get_active() == False:
            self.debug_checkbutton.set_active(False)

    def on_cpu_checkbutton_toggled(self, widget):
        # print 'on_cpu_checkbutton_toggled'
        if self.cpu_checkbutton.get_active() == False:
            self.cpu_comboboxtext.set_sensitive(False)
        else:
            self.cpu_comboboxtext.set_sensitive(True)

    def on_mem_checkbutton_toggled(self, widget):
        # print 'on_mem_checkbutton_toggled'
        if self.mem_checkbutton.get_active() == False:
            self.mem_spinbutton.set_sensitive(False)
        else:
            self.mem_spinbutton.set_sensitive(True)

    def on_debug_checkbutton_toggled(self, widget):
        if self.debug_checkbutton.get_active() == False:
            self.debug_port_checkbutton.set_sensitive(False)
            self.debug_port_comboboxtext.set_sensitive(False)
            self.baud_rate_checkbutton.set_sensitive(False)
            self.baud_rate_comboboxtext.set_sensitive(False)
        else:
            self.debug_port_checkbutton.set_sensitive(True)
            self.debug_port_checkbutton.set_active(True)
            self.debug_port_comboboxtext.set_sensitive(True)
            self.baud_rate_checkbutton.set_sensitive(True)
            self.baud_rate_checkbutton.set_active(True)
            self.baud_rate_comboboxtext.set_sensitive(True)
            self.more_boot_log_checkbutton.set_active(True)

    def get_new_cmdline_linux_default(self):
        basic_str = "\""
        advanced_str = ""
        if "GRUB_CMDLINE_LINUX_DEFAULT" in self.grub_env:
            for cmd_line in self.grub_env["GRUB_CMDLINE_LINUX_DEFAULT"][1:-1].split():
                if (not cmd_line == "quiet") and (not cmd_line == "splash") and (not cmd_line == "nosmp") and (not cmd_line.startswith("maxcpus=")) and (not cmd_line.startswith("mem=")) and (not cmd_line.startswith("loglevel=")) and (not cmd_line.startswith("console=")):
                    advanced_str += (cmd_line + " ")
            # print basic_str
        if self.bootlog_checkbutton.get_active() == False:
            basic_str += "quiet "
        if self.splash_checkbutton.get_active() == False:
            basic_str += "splash "
        if self.more_boot_log_checkbutton.get_active() == True:
            basic_str += "loglevel=7 "
        if self.cpu_checkbutton.get_active() == True:
            if not self.cpu_comboboxtext.get_active_text() == None:
                basic_str += "maxcpus=" + self.cpu_comboboxtext.get_active_text() + " "
        if self.mem_checkbutton.get_active() == True:
            basic_str += "mem=" + str(self.mem_spinbutton.get_value_as_int()) + "M "
        if self.debug_checkbutton.get_active():
            basic_str += "console=tty0 "
            basic_str += "console=" + self.debug_port_comboboxtext.get_active_text() + "," + self.baud_rate_comboboxtext.get_active_text() + "n8 "
        basic_str += advanced_str
        if basic_str.endswith(" "):
            basic_str = basic_str[:-1]
        basic_str += "\""
        return basic_str

    def save_grub_env(self, update_kernel=True, update_cmdline=True):
        try:
            write_data = ""
            orig_cmd_line = orig_default = None
            if not os.path.exists(self.grub_cfg_file):
                if not os.path.exists(self.grub_cfg_dir):
                    os.mkdir(self.grub_cfg_dir)
            elif os.path.isdir(self.grub_cfg_file):
                os.rmdir(self.grub_cfg_file)
            else:
                for line in open( self.grub_cfg_file).read().split('\n'):
                    if not line == '':
                        if line.startswith("GRUB_CMDLINE_LINUX_DEFAULT"):
                            orig_cmd_line = line + "\n"
                        elif line.startswith("GRUB_DEFAULT"):
                            orig_default = line + "\n"
                        else:
                            write_data += line + "\n"

            if update_cmdline:
                cmd_line = "GRUB_CMDLINE_LINUX_DEFAULT="+self.get_new_cmdline_linux_default()+"\n"
            elif orig_cmd_line is not None:
                cmd_line = orig_cmd_line
            else:
                cmd_line = None

            if update_kernel and self.storage.get_choosen():
                if self.storage.get_kernel_submenu_id() is not None:
                    default = "GRUB_DEFAULT=\"" + self.storage.get_kernel_submenu_id() + ">" + self.storage.get_kernel_menu_id() + "\"\n"
                else:
                    default = "GRUB_DEFAULT=\"" + self.storage.get_kernel_menu_id() + "\"\n"
            elif orig_default is not None:
                default = orig_default
            else:
                default = None

            if cmd_line is not None:
                write_data += cmd_line
                self.cmd_line = cmd_line[27:]
            if default is not None:
                write_data += default
                self.boot_kernel = default[13:]

            # print write_data
            open( self.grub_cfg_file, 'w').write(write_data)

            return True
        except Exception as e:
            return False

    def on_reset_button_clicked(self, widget):
        GLib.idle_add(self.reset_ui, widget)
        # cmdline_linux_default_line = self.get_new_cmdline_linux_default()
        # self.save_grub_env(cmdline_linux_default_line)

    def on_ok_button_clicked(self, widget):
        GLib.idle_add(self.on_ok_button_clicked_handler, widget)
        # self.on_ok_button_clicked_handler(widget)

    def on_ok_button_clicked_handler(self, widget):
        self.cmd_line = None
        self.boot_kernel = None
        self.save_grub_env()
        update_boot_grub_cfg(grub_env=self.grub_env, menu_id=self.boot_kernel, cmd_linux_defalut_line=self.cmd_line, update_menu=True, update_cmdline=True)
        self.do_close()

    def show(self):
        self.boot_mgr.set_sensitive_for_child_window(False)
        self.window.present()
        self.window.show_all()
        self.window.set_keep_above(True)

    def close(self, widget, event):
        self.do_close()

    def do_close(self):
        self.boot_mgr.set_sensitive_for_child_window(True)
        self.window.destroy()
        self.destroy_flag = True
        return True

    def on_cancel_button_clicked(self, widget):
        self.do_close()
        return True

class Kernel_Manager(Gtk.EventBox):
    def __init__(self, box, storage, kernel_json):
        Gtk.EventBox.__init__(self)
        self.set_size_request(-1, 26)
        self.box = box
        self.storage = storage
        self.kernel_json = kernel_json
        self.active_status = False
        self.connect("enter-notify-event", self.eventbox_enter_color)
        self.connect("leave-notify-event", self.eventbox_leave_color)
        self.connect("button-press-event", self.eventbox_active_color)
        self.set_kernel_info()

    def set_kernel_info(self):
        self.label = Gtk.Label(self.kernel_json["kernel"])
        self.label.set_margin_left(10)
        self.label.set_xalign(0)
        self.add(self.label)

    def set_active(self, status):
        if status == True:
            self.active_status = True
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(45568, 57600, 65535))
            self.storage.set_kernel_name(self.kernel_json["kernel"])
            self.storage.set_kernel_menu_id(self.kernel_json["menu_id"])
            self.storage.set_kernel_submenu_id(self.kernel_json["submenu_id"])
            self.storage.set_choosen(True)
            for eventbox in self.box:
                if eventbox != self:
                    eventbox.set_active(False)
        elif status == False:
            self.active_status = False
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(65535, 65535, 65535))

    def eventbox_enter_color(self, widget, event):
        if self.active_status == False:
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(219*256, 241*256, 255*256))

    def eventbox_leave_color(self, widget, event):
        if self.active_status == False:
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(65535,65535,65535))

    def eventbox_active_color(self, widget, event):
        self.set_active(True)

class Kernel_Storage(object):
    def __init__(self, kernel_name=None, kernel_menu_id=None,kernel_submenu_id=None):
        self.kernel_name = kernel_name
        self.kernel_menu_id = kernel_menu_id
        self.kernel_submenu_id = kernel_submenu_id
        self.choose_flag = False

    def set_kernel_name(self, kernel_name):
        self.kernel_name = kernel_name

    def set_kernel_menu_id(self, kernel_menu_id):
        self.kernel_menu_id = kernel_menu_id

    def set_kernel_submenu_id(self, kernel_submenu_id):
        self.kernel_submenu_id = kernel_submenu_id

    def set_choosen(self, status):
        if status == True:
            self.choose_flag = True
        elif status == False:
            self.choose_flag = False

    def get_kernel_name(self):
        return self.kernel_name

    def get_kernel_menu_id(self):
        return self.kernel_menu_id

    def get_kernel_submenu_id(self):
        return self.kernel_submenu_id

    def get_choosen(self):
        return self.choose_flag

if __name__ == "__main__":
    boot_mgr = Advanced_Option_UI(".")
    boot_mgr.show()
    Gtk.main()

