#!/usr/bin/python
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import re
import dbus
import dbus.service
import time
import os
import gi
import subprocess
import thread
import hashlib
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GLib,Gio,Gdk,GObject
from gi.repository.GdkPixbuf import Pixbuf,PixbufAnimation
from dbus.mainloop.glib import DBusGMainLoop
from firmware_manager import *
from advanced_options import *
from grub_lib import *
from basic_lib import *
from uefi_lib import *
from warning import *

class Boot_Manager_UI(object):

    def __init__(self, main_dir):
        if os.getuid() != 0:
            exit(1)
        self.main_dir = main_dir
        self.uidir = main_dir + "/ui"
        self.img_dir = main_dir + "/pixmaps"
        self.grub_cfg_dir = "/etc/default/grub.d/"
        self.grub_cfg_file = "/etc/default/grub.d/z_boot_mgr_grub.cfg"
        import gettext as gettext1
        gettext1.bindtextdomain('boot_manager', "/usr/share/locale")
        gettext1.textdomain('boot_manager')
        self._ = gettext1.gettext
        self.refresh_info = self._("Searching other operating system.\nClose it later.")
        provider = Gtk.CssProvider()
        provider.load_from_data(b'''\
            #os_scrolledwindow,
            #mainwindow {
                background: #ffffff;
            }
            GtkButton {
                border: none;
                border-radius: 0px;
            }
            #choice_label,
            #choosen_label,
            #timeout_label,
            #time_unit_label,
            #fastboot_label {
                color: #333333;
                font-size: 14px;
            }
            #choice_label:insensitive,
            #choosen_label:insensitive,
            #timeout_label:insensitive,
            #time_unit_label:insensitive,
            #fastboot_label:insensitive {
                color: #999999;
            }
            #refresh_button,
            #advanced_option_button{
                border: none;
                border-radius: 0px;
                border-color: #ffffff;
                color: #007ed2;
                font-size: 14px;
                background: #fff;
            }
            #refresh_button:insensitive {
                color: #999999;
            }
            #bios_setup_button,
            #apply_button {
                border: none;
                border-radius: 0px;
                color: #333333;
                font-size: 16px;
                background: #dddddd;
            }
            #bios_setup_button:hover,
            #apply_button:hover {
                background: #e2e2e2;
            }
            #bios_setup_button:active,
            #apply_button:active {
                background: #c7c7c7;
            }
            #timeout_entry {
                border-radius: 0px;
            }
            #notify_label {
                color: #999999;
                font-size: 12px;
            }
            ''')
        Gtk.StyleContext.add_provider_for_screen(
            Gdk.Screen.get_default(),
            provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

        self.builder = Gtk.Builder()
        self.builder.add_from_file(self.uidir + "/mainwindow.ui")
        self.builder.connect_signals(self)
        self.window = self.builder.get_object("mainwindow")
        self.window.set_title(self._("Boot Manager"))
        self.window.set_name("mainwindow")
        self.window.connect("delete-event", self.close)
        self.window.set_icon_from_file(self.img_dir+"/boot-manager-logo.svg")
        # label
        self.choice_label = self.builder.get_object("choice_label")
        self.choice_label.set_name("choice_label")
        self.choice_label.set_markup(self._("Please select default OS"))
        self.os_grid = self.builder.get_object("os_grid")
        self.os_scrolledwindow = self.builder.get_object("os_scrolledwindow")
        self.os_scrolledwindow.set_name("os_scrolledwindow")
        self.choosen_label = self.builder.get_object("choosen_label")
        self.choosen_label.set_name("choosen_label")
        self.choosen_label.set_markup(self._("You havent choose an operating system as default boot operating system"))

        self.timeout_label = self.builder.get_object("timeout_label")
        self.timeout_label.set_name("timeout_label")
        self.timeout_label.set_markup(self._("Timeout:"))
        self.timeout_entry = self.builder.get_object("timeout_entry")
        self.timeout_entry.set_name("timeout_entry")

        self.time_unit_label = self.builder.get_object("time_unit_label")
        self.time_unit_label.set_name("time_unit_label")
        self.time_unit_label.set_markup(self._("seconds"))
        # button
        self.advanced_option_button = self.builder.get_object("advanced_option_button")
        self.advanced_option_button.set_name("advanced_option_button")
        self.advanced_option_button.set_label(self._("Advanced options..."))
        self.refresh_button = self.builder.get_object("refresh_button")
        self.refresh_button.set_name("refresh_button")
        self.refresh_button.set_label(self._("Search other operating system..."))
        self.refresh_button.connect("clicked", self.refresh_os_info)
        self.refresh_flag = False
        # label
        self.fastboot_label = self.builder.get_object("fastboot_label")
        self.fastboot_label.set_name("fastboot_label")
        self.fastboot_label.set_markup(self._("fastboot"))
        # switchbutton
        self.fastboot_switchbutton = self.builder.get_object("fastboot_switchbutton")
        self.fastboot_switchbutton.set_name("fastboot_switchbutton")

        self.bios_setup_button = self.builder.get_object("bios_setup_button")
        self.bios_setup_button.set_name("bios_setup_button")
        self.bios_setup_button.set_label(self._("Reboot to BIOS Setup"))

        self.notify_label = self.builder.get_object("notify_label")
        self.notify_label.set_name("notify_label")
        self.notify_label.set_markup(self._("Tips: Some functions is limit by UEFI,so that you may can't use some of them."))

        self.apply_button = self.builder.get_object("apply_button")
        self.apply_button.set_name("apply_button")
        self.apply_button.set_label(self._("apply"))

        self.firmwaremgr = None
        self.advanced_option_object = None

        # self.window.set_size_request(200, 150)
        self.window.set_position(Gtk.WindowPosition.CENTER)
        # self.window.connect("delete-event", self.hide)

        GLib.idle_add(self.init_default_ui)


    def refresh_os_info(self, wdiget):
        def do_refresh_os_info(self):
            self.refresh_flag = True
            GLib.idle_add(self.window.set_sensitive, False)
            for widget in self.os_grid.get_children():
                try:
                    widget.destroy()
                except Exception as e:
                    pass
            # loading info
            loading_image = Gtk.Image()
            loading_image_name = os.path.join( self.img_dir, "loading.gif")
            loading_pixbuf = PixbufAnimation.new_from_file( loading_image_name)
            loading_image.set_from_animation(loading_pixbuf)
            loading_image.set_margin_top(60)
            self.os_grid.attach(loading_image, 0, 0, 1, 1)
            loading_image.show()
            refreshing_label = Gtk.Label()
            refreshing_label.set_text(self._("Searching..."))
            refreshing_label.set_name("refreshing_label")
            self.os_grid.attach(refreshing_label, 0, 1, 1, 1)
            refreshing_label.set_hexpand(True)
            refreshing_label.set_xalign(0.5)
            refreshing_label.show()
            try:
                os.system("sudo update-grub")
                # delete loading info
                loading_image.destroy()
                refreshing_label.destroy()
                GLib.idle_add(self.init_os_info)
            except Exception as e:
                pass
            finally:
                GLib.idle_add(self.window.set_sensitive, True)
                pass
            self.refresh_flag = False
        thread.start_new_thread(do_refresh_os_info, (self,))


    def init_default_ui(self):
        # timeout
        self.timeout_entry.set_max_length(10)
        self.grub_env = get_grub_env_from_file()
        if "GRUB_TIMEOUT" in self.grub_env:
            timeout = self.grub_env["GRUB_TIMEOUT"]
            if timeout.startswith("\"") or timeout.startswith("\'"):
                timeout = timeout[1:]
            if timeout.endswith("\"") or timeout.endswith("\'"):
                timeout = timeout[:-1]
        else:
            timeout = "5"

        if timeout == "-1":
            timeout = ""
        elif timeout == "0":
            timeout = "10"
        self.old_timeout = self.init_timeout = timeout
        self.timeout_entry.set_text(timeout)
        self.init_os_info()
        # RebootToFirmwareSetup
        self.firmwaresetup_flag = self.canreboottofirmwaresetup()
        if not self.firmwaresetup_flag:
            self.bios_setup_button.set_sensitive(False)
        # fastboot
        self.init_fastboot_state()

    def init_os_info(self):
        self.storage = OS_Storage()
        default_id = self.grub_env["GRUB_DEFAULT"].split(">")[0]
        if default_id.startswith("\'") or default_id.startswith("\""):
            default_id = default_id[1:]
        if default_id.endswith("\'") or default_id.endswith("\""):
            default_id = default_id[:-1]
        i = j = 0
        self.os_grid.set_column_spacing(10)
        self.os_grid.set_row_spacing(10)
        self.current_os_info = current_os_info = get_10_linux_basic_info_from_cfg()
        # print current_os_info

        if not ("root" in current_os_info):
            '''
            The member of "root" maybe not in current_os_info 
            that build by func of get_10_linux_basic_info_from_cfg().
            This is not what we want. 
            '''
            current_os_info["root"] = ""

        os_obj = OS_Manager(self.img_dir, os_grid=self.os_grid, label=self.choosen_label, \
                        storage=self.storage, advanced_option_button=self.advanced_option_button, \
                        get_text=self._, menu_id=current_os_info["menu_id"], title=current_os_info["title"], \
                        root=current_os_info["root"], current_os_flag=True, show_disk=False)
        if default_id == "0" or current_os_info["menu_id"] == default_id \
                or (("submenu_id" in current_os_info) and (current_os_info["submenu_id"] == default_id)):
            os_obj.set_active(True)
        self.os_grid.attach(os_obj,i,j,1,1)
        i += 1
        os_obj.show_all()
        other_os_basic_infos = get_30_os_prober_basic_info_from_cfg()
        if len(other_os_basic_infos) == 0:
            self.no_other_os_flag = True
            self.timeout_label.set_sensitive(False)
            self.time_unit_label.set_sensitive(False)
            self.timeout_entry.set_sensitive(False)
        else:
            self.no_other_os_flag = False
        for os_basic_info in other_os_basic_infos:
            # print os_basic_info
            if not ("root" in os_basic_info):
                '''
                The member of "root" maybe not in os_basic_info.
                This is not what we want. 
                '''
                os_basic_info["root"] = ""

            os_obj = OS_Manager(self.img_dir, os_grid=self.os_grid, label=self.choosen_label, \
                            storage=self.storage, advanced_option_button=self.advanced_option_button, \
                            get_text=self._, menu_id=os_basic_info["menu_id"], title=os_basic_info["title"], \
                            root=os_basic_info["root"])
            if default_id == os_basic_info["menu_id"]:
                os_obj.set_active(True)
            self.os_grid.attach(os_obj,i,j,1,1)
            os_obj.show_all()
            if i < 7:
                i += 1
            else:
                i = 0
                j += 1

    def on_timeout_entry_changed(self, widget):
        try:
            # if timeout is int
            if self.timeout_entry.get_text().isdigit() or self.timeout_entry.get_text() == "":
                if self.timeout_entry.get_text() == "0":
                    self.timeout_entry.set_text("1")
                    self.old_timeout = "1"
                else:
                    self.old_timeout = self.timeout_entry.get_text()
            else:
                self.timeout_entry.set_text(self.old_timeout)
        except Exception as e:
            self.timeout_entry.set_text(self.old_timeout)

    def set_sensitive_for_child_window(self, state):
        if state == False:
            self.choice_label.set_sensitive(False)
            self.os_scrolledwindow.set_sensitive(False)
            # for eventbox in self.os_grid:
            #     eventbox.set_sensitive(False)
            self.choosen_label.set_sensitive(False)
            if not self.no_other_os_flag:
                self.timeout_label.set_sensitive(False)
                self.time_unit_label.set_sensitive(False)
                self.timeout_entry.set_sensitive(False)
            self.advanced_option_button.set_sensitive(False)
            if self.fastboot_can_use_flag == True:
                self.fastboot_label.set_sensitive(False)
                self.fastboot_switchbutton.set_sensitive(False)
            self.bios_setup_button.set_sensitive(False)
            self.apply_button.set_sensitive(False)
        elif state == True:
            self.choice_label.set_sensitive(True)
            self.os_scrolledwindow.set_sensitive(True)
            # for eventbox in self.os_grid:
            #     eventbox.set_sensitive(True)
            self.choosen_label.set_sensitive(True)
            if self.no_other_os_flag == False:
                self.timeout_label.set_sensitive(True)
                self.time_unit_label.set_sensitive(True)
                self.timeout_entry.set_sensitive(True)
            self.advanced_option_button.set_sensitive(True)
            if self.fastboot_can_use_flag == True:
                self.fastboot_label.set_sensitive(True)
                self.fastboot_switchbutton.set_sensitive(True)
            if self.firmwaresetup_flag:
                self.bios_setup_button.set_sensitive(True)
            self.apply_button.set_sensitive(True)

    def init_fastboot_state(self):
        if get_fastboot_can_use() == False:
            self.fastboot_can_use_flag = False
            self.fastboot_state = False
            self.fastboot_label.set_sensitive(False)
            self.fastboot_switchbutton.set_sensitive(False)
        else:
            self.fastboot_can_use_flag = True
            self.fastboot_state = self.orig_fastboot_state = get_fastboot_state()
            self.fastboot_switchbutton.set_state(self.fastboot_state)
            self.on_fastboot_switchbutton_state_set( self.fastboot_switchbutton, self.fastboot_state)

    def on_fastboot_switchbutton_state_set(self, widget, event):
        if event == True:
        # if self.fastboot_switchbutton.get_state() == True:
            self.fastboot_state = True
            self.choice_label.set_sensitive(False)
            self.advanced_option_button.set_sensitive(False)
            self.os_scrolledwindow.set_sensitive(False)
            self.choosen_label.set_sensitive(False)
            if not self.no_other_os_flag:
                self.timeout_label.set_sensitive(False)
                self.time_unit_label.set_sensitive(False)
                self.timeout_entry.set_sensitive(False)
            self.bios_setup_button.set_sensitive(False)
            for eventbox in self.os_grid:
                eventbox.set_sensitive(False)
        else:
            self.fastboot_state = False
            self.choice_label.set_sensitive(True)
            self.advanced_option_button.set_sensitive(True)
            self.os_scrolledwindow.set_sensitive(True)
            self.choosen_label.set_sensitive(True)
            if not self.no_other_os_flag:
                self.timeout_label.set_sensitive(True)
                self.time_unit_label.set_sensitive(True)
                self.timeout_entry.set_sensitive(True)
            if self.firmwaresetup_flag:
                self.bios_setup_button.set_sensitive(True)
            for eventbox in self.os_grid:
                eventbox.set_sensitive(True)

    def on_apply_button_clicked(self, widget):
        GLib.idle_add(self.on_apply_button_clicked_handle, widget)

    def get_mount_info(self):
        self.efi_part = self.efi_part_name = self.efi_part_id = None
        self.root_part = self.root_part_name = self.root_part_id = None
        try:
            mount_infos = subprocess.check_output("mount").split("\n")
            for mount_info in mount_infos:
                if "/boot/efi" in mount_info:
                    self.efi_part = mount_info.split()[0]
                    self.efi_part_name = re.split(r"[0-9]", self.efi_part)[0]
                    self.efi_part_id = self.efi_part[len(self.efi_part_name):]
                if "on / " in mount_info:
                    self.root_part = mount_info.split()[0]
                    self.root_part_name = re.split(r"[0-9]", self.root_part)[0]
                    self.root_part_id = self.root_part[len(self.root_part_name):]
        except Exception as e:
            pass

    def on_apply_button_clicked_handle(self, widget):
        if self.fastboot_can_use_flag == True:
            if self.fastboot_state == True:
                if self.orig_fastboot_state == True:
                    # do nothing and return
                    self.do_close()
                    return
                elif self.orig_fastboot_state == False:
                    # set fastboot on
                    set_fastboot_state(True)
                    self.do_close()
                    return
            elif self.fastboot_state == False:
                if self.orig_fastboot_state == True:
                    # set fastboot off and go to grub, no return
                    set_fastboot_state(False)
                # elif self.orig_fastboot_state == False:
                #     # set nothing and go to grub, no return
                #     pass
        self.apply_for_grub()
        self.do_close()

    def apply_for_grub(self):
        update_dafault_menu_flag = False
        if self.storage.get_choosen():
            tmp_grub_env = get_grub_env_from_file()
            if "GRUB_DEFAULT" in tmp_grub_env:
                old_default_menu = tmp_grub_env["GRUB_DEFAULT"]
            else:
                old_default_menu = "0"
            if old_default_menu.startswith("\"") or old_default_menu.startswith("\'"):
                old_default_menu = old_default_menu[1:]
            if old_default_menu.endswith("\"") or old_default_menu.endswith("\'"):
                old_default_menu = old_default_menu[:-1]
            old_default_menu = old_default_menu.split(">")[0]

            # if choose other os
            if self.storage.get_current_os_flag():
                update_dafault_menu_flag = True
                # print "current os"
                #if (not old_default_menu == self.current_os_info["menu_id"]) and \
                #    ("submenu_id" in self.current_os_info) and \
                #    (not old_default_menu == self.current_os_info["submenu_id"]):
                #    update_dafault_menu_flag = True
            else:
                if not (self.storage.get_menu_id() == old_default_menu):
                    update_dafault_menu_flag = True
        update_timeout_flag = False
        current_timeout = self.timeout_entry.get_text()
        if (not self.init_timeout == current_timeout):
            update_timeout_flag = True

        self.default_line = None
        self.timeout_line = None
        self.save_grub_env(update_dafault_menu_flag, update_timeout_flag)
        update_boot_grub_cfg(self.grub_env, menu_id=self.default_line , timeout=self.timeout_line, update_menu=update_dafault_menu_flag, update_timeout=update_timeout_flag)
        self.get_mount_info()
        try:
            # if legacy
            if not get_system_boot_state() == "EFI":
                if os.path.isfile("/boot/grub/i386-pc/core.img") and os.path.isfile("/boot/grub/i386-pc/boot.img"):
                    os.system("grub-bios-setup -c i386-pc/core.img -b i386-pc/boot.img /dev/sda")
                else:
                    if self.root_part_name is not None:
                        os.system("grub-install " + self.root_part_name)
            # if efi
            else:
                if self.get_efi_boot_info() == False:
                    if self.root_part_name is not None:
                        os.system("grub-install " + self.root_part_name)
                        if os.path.isdir("/usr/share/boot-manager/grub.cfg"):
                            os.rmdir("/usr/share/boot-manager/grub.cfg")
                        shutil.copyfile("/boot/efi/EFI/cdos/grub.cfg", "/usr/share/boot-manager/grub.cfg")
        except Exception as e:
            pass

    def get_efi_boot_info(self):
        boot_list = subprocess.check_output( ['efibootmgr', '-v']).split('\n')
        bootorder = None
        boot_num = None
        for line in boot_list:
            if line.startswith("BootOrder"):
                bootorder = line.split()[1]
            if (len(line.split()) > 1) and (line.split()[1] == "cdos") and ("\\EFI\\cdos\\shimx64.efi" in line):
                boot_num = line.split()[0]
                if boot_num.endswith("*"):
                    boot_num = boot_num[4:-1]
                else:
                    boot_num = boot_num[4:]
        # all file is exist,update bootnum
        if os.path.isfile("/boot/efi/EFI/cdos/shimx64.efi") and os.path.isfile("/boot/efi/EFI/cdos/grubx64.efi") and \
            os.path.isfile("/usr/share/boot-manager/grub.cfg"):
            try:
                shutil.copyfile("/usr/share/boot-manager/grub.cfg", "/boot/efi/EFI/cdos/grub.cfg")
            except Exception as e:
                return False
        elif os.path.isfile("/usr/share/boot-manager/grub.cfg") and \
            os.path.isfile("/usr/lib/shim/shimx64.efi.signed") and os.path.isfile("/usr/lib/grub/x86_64-efi-signed/grubx64.efi.signed"):
            try:
                shutil.copyfile("/usr/share/boot-manager/grub.cfg", "/boot/efi/EFI/cdos/grub.cfg")
                for efi_file_name in os.listdir("/usr/lib/shim/"):
                    if efi_file_name.endswith(".efi.signed") and os.path.isfile("/usr/lib/shim/" + efi_file_name):
                        shutil.copyfile("/usr/lib/shim/" + efi_file_name, "/boot/efi/EFI/cdos/" + efi_file_name[:-7])
                shutil.copyfile("/usr/lib/grub/x86_64-efi-signed/grubx64.efi.signed", "/boot/efi/EFI/cdos/grubx64.efi")
            except Exception as e:
                return False
        else:
            return False

        # try to remove old bootnum
        if (boot_num is not None):
            try:
                os.system("efibootmgr -B -b "+ boot_num)
            except Exception as e:
                pass
        # add new bootnum
        try:
            if (self.efi_part_name is None) or (self.efi_part_id is None):
                return False
            if not (os.system("efibootmgr -c -g -d " + self.efi_part_name + " -p " + self.efi_part_id + " -L \"cdos\" -l /EFI/cdos/shimx64.efi") == 0):
                return False
        except Exception as e:
            return False
        return True

    def get_md5(self, file_name):
        try:
            md5obj = hashlib.md5()
            md5obj.update(open(file_name,"rb").read())
            return md5obj.hexdigest()
        except Exception as e:
            return None

    def save_grub_env(self, update_dafault_menu_flag=False, update_timeout_flag=False):
        try:
            write_data = ""
            orig_timeout = None
            if not os.path.exists(self.grub_cfg_file):
                if not os.path.exists(self.grub_cfg_dir):
                    os.mkdir(self.grub_cfg_dir)
            elif os.path.isdir(self.grub_cfg_file):
                os.rmdir(self.grub_cfg_file)
            else:
                for line in open( self.grub_cfg_file).read().split('\n'):
                    if not line == '':
                        if line.startswith("GRUB_TIMEOUT"):
                            orig_timeout = line + "\n"
                        elif line.startswith("GRUB_DEFAULT"):
                            orig_default = line + "\n"
                        else:
                            write_data += line + "\n"
            if update_dafault_menu_flag:
                default_line = "GRUB_DEFAULT=\"" + self.storage.get_menu_id() + "\"\n"
            elif orig_default is not None:
                default_line = orig_default
            else:
                default_line = None
            if update_timeout_flag:
                timeout_str = self.timeout_entry.get_text()
                if timeout_str == "":
                    timeout_str = "-1"
                timeout_line = "GRUB_TIMEOUT="+timeout_str+"\n"
            elif orig_timeout is not None:
                timeout_line = orig_timeout
            else:
                timeout_line = None

            if default_line is not None:
                write_data += default_line
                self.default_line = default_line[13:]
            if timeout_line is not None:
                write_data += timeout_line
                self.timeout_line = timeout_line[13:]

            open( self.grub_cfg_file, 'w').write(write_data)
            return True
        except Exception as e:
            return False

    def canreboottofirmwaresetup(self):
        try:
            # if legacy
            if not get_system_boot_state() == "EFI":
                return False
            # efi
            if biosinfirmwaresetupblacklist():
                return False
            firmware_setup_dbus_name = "org.freedesktop.login1"
            firmware_setup_dbus_path = "/org/freedesktop/login1"
            firmware_setup_dbus_iface = "org.freedesktop.login1.Manager"
            self.firmware_setup_dbus_obj = dbus.SystemBus().get_object( firmware_setup_dbus_name, firmware_setup_dbus_path)
            self.firmware_setup_dbus_iface_obj = dbus.Interface( self.firmware_setup_dbus_obj, firmware_setup_dbus_iface)
            if not self.firmware_setup_dbus_iface_obj.CanRebootToFirmwareSetup() == "yes":
                return False
            return True
        except Exception as e:
            return False

    def setreboottofirmwaresetup(self):
        try:
            self.firmware_setup_dbus_iface_obj.SetRebootToFirmwareSetup(True)
            return True
        except Exception as e:
            return False

    def show(self):
        self.window.present()
        self.window.show_all()

    def close(self,  widget, event):
        if self.refresh_flag == True:
            warning_dialog = WarningDialog(self, self.uidir, self.img_dir)
            warning_dialog.set_label(self.refresh_info)
            warning_dialog.show()
            return True
        self.do_close()

    def do_close(self):
        if (self.firmwaremgr is not None) and (not self.firmwaremgr.destroy_flag):
            self.firmwaremgr.do_close()
        if (self.advanced_option_object is not None) and (not self.advanced_option_object.destroy_flag):
            self.advanced_option_object.do_close()
        self.window.destroy()
        exit(0)

    def on_advanced_option_button_clicked(self, unused_widget):
        self.advanced_option_object = Advanced_Option_UI(self.uidir, self, self.grub_cfg_dir, self.grub_cfg_file)
        self.advanced_option_object.show()
        return True

    def on_bios_setup_button_clicked(self, unused_widget):
        self.firmwaremgr = FirmwareManager(uidir=self.uidir, boot_manager=self)
        self.firmwaremgr.show()

class OS_Manager(Gtk.EventBox):
    def __init__(self, img_dir, os_grid, label, storage, advanced_option_button, get_text, menu_id, submenu_id=None, title="", root="", current_os_flag=False, show_disk=True):
        Gtk.EventBox.__init__(self)
        import gettext as gettext1
        gettext1.bindtextdomain('boot_manager', "/usr/share/locale")
        gettext1.textdomain('boot_manager')
        self._ = gettext1.gettext
        self.set_size_request(80, 100)
        self.connect("enter-notify-event", self.eventbox_enter_color)
        self.connect("leave-notify-event", self.eventbox_leave_color)
        self.connect("button-press-event", self.eventbox_active_color)
        self.active_status = False
        self.os_grid = os_grid
        self.label = label
        self.storage = storage
        self.advanced_option_button = advanced_option_button
        self.get_text = get_text
        self.current_os_flag = current_os_flag
        self.menu_id = menu_id
        self.title = title
        self.root = root
        if img_dir.endswith("/"):
            self.img_dir = img_dir
        else:
            self.img_dir = img_dir + "/"
        if not current_os_flag:
            self.disk_name = title.split()[-1][:-1].split("/")[-1]
            # print self.disk_name
        self.show_os_info(show_disk)

    def show_os_info(self, show_disk=True):
        box = Gtk.VBox( spacing=0)
        title = self.title.lower()
        self.image = Gtk.Image()
        self.image.set_margin_left(10)
        self.image.set_margin_right(10)
        self.image.set_margin_top(10)
        self.image.set_margin_bottom(10)
        image_wigth = 48
        image_height = 48
        if ("cdos" in title) or ("核高基" in title) or ("基础版" in title):
            self.image_name = self.img_dir+"cdos.png"
            self.sensitive_image_name = self.img_dir+"cdos_insensitive.png"
            os_name = self._("CDOS")
        elif ("nfs" in title) or ("方德" in title):
            self.image_name = self.img_dir+"nfs.png"
            self.sensitive_image_name = self.img_dir+"nfs_insensitive.png"
            os_name = "NFS"
        elif ("debian" in title):
            self.image_name = self.img_dir+"debian.png"
            self.sensitive_image_name = self.img_dir+"debian_insensitive.png"
            os_name = "debian"
        elif ("deepin" in title):
            self.image_name = self.img_dir+"deepin.png"
            self.sensitive_image_name = self.img_dir+"deepin_insensitive.png"
            os_name = "Deepin"
        elif ("windows" in title):
            self.image_name = self.img_dir+"windows.png"
            self.sensitive_image_name = self.img_dir+"windows_insensitive.png"
            os_name = "Windows"
        elif ("linuxmint" in title) or ("mint" in title):
            self.image_name = self.img_dir+"linuxmint.png"
            self.sensitive_image_name = self.img_dir+"linuxmint_insensitive.png"
            os_name = "Linux Mint"
        elif ("fedora" in title):
            self.image_name = self.img_dir+"fedora.png"
            self.sensitive_image_name = self.img_dir+"fedora_insensitive.png"
            os_name = "Fedora"
        elif ("centos" in title):
            self.image_name = self.img_dir+"centos.png"
            self.sensitive_image_name = self.img_dir+"centos_insensitive.png"
            os_name = "CentOS"
        elif ("ubuntu" in title):
            self.image_name = self.img_dir+"ubuntu.png"
            self.sensitive_image_name = self.img_dir+"ubuntu_insensitive.png"
            os_name = "Ubuntu"
        else:
            self.image_name = self.img_dir+"other.png"
            self.sensitive_image_name = self.img_dir+"other_insensitive.png"
            os_name = self.title.split()[0]

        self.pixbuf = Pixbuf.new_from_file_at_scale(self.image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)
        self.sensitive_pixbuf = Pixbuf.new_from_file_at_scale(self.sensitive_image_name, width=image_wigth, height=image_height, preserve_aspect_ratio=False)

        self.image.set_from_pixbuf(self.pixbuf)
        box.pack_start( self.image, False, False, 0)
        # if show_disk == False:
        #     box.pack_start( Gtk.Label(self.title.split()[0]), False, True, 0)
        # elif show_disk == True:
        #     box.pack_start( Gtk.Label(self.title.split()[0] + "\n" + self.disk_name), False, True, 0)
        box.pack_start( Gtk.Label(os_name), False, True, 0)
        # box.pack_start( Gtk.Label(title.split()[0] + "\n" + self.root), True, True, 0)
        self.add(box)

    def set_sensitive(self, status):
        Gtk.EventBox.set_sensitive(self, status)
        # dbf1ff hover
        # e5e5e5 insensitive
        if status == False and self.active_status == True:
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(229*256, 229*256, 229*256))
        # B2E1FF active
        elif status == True and self.active_status == True:
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(45568, 57600, 65535))
        # FFFFFF
        # elif status == True and self.active_status == False:
        else:
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(65535, 65535, 65535))

        if status == False:
            self.image.set_from_pixbuf(self.sensitive_pixbuf)
        elif status == True:
            self.image.set_from_pixbuf(self.pixbuf)

    def set_active(self, status):
        if status == True:
            self.active_status = True
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(45568, 57600, 65535))
            self.storage.set_menu_id(self.menu_id)
            self.storage.set_choosen(True)
            self.label.set_markup(self.get_text("You choose {RELEASE} as default boot operating system").replace("{RELEASE}", self.title))
            if self.current_os_flag:
                self.advanced_option_button.set_sensitive(True)
                self.storage.set_current_os_flag(True)
            else:
                self.advanced_option_button.set_sensitive(False)
                self.storage.set_current_os_flag(False)
            for eventbox in self.os_grid:
                if eventbox != self:
                    eventbox.set_active(False)
        elif status == False:
            self.active_status = False
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(65535, 65535, 65535))

    def eventbox_enter_color(self, widget, event):
        if self.active_status == False:
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(219*256, 241*256, 255*256))

    def eventbox_leave_color(self, widget, event):
        if self.active_status == False:
            self.modify_bg(Gtk.StateType.NORMAL, Gdk.Color(65535,65535,65535))

    def eventbox_active_color(self, widget, event):
        self.set_active(True)

class OS_Storage(object):
    def __init__(self, os_name=None, menu_id=None, submenu_id=None, current_os_flag = None):
        self.os_name = os_name
        self.menu_id = menu_id
        self.submenu_id = submenu_id
        self.current_os_flag = current_os_flag
        self.choose_flag = False

    def set_os_name(self, os_name):
        self.os_name = os_name

    def set_menu_id(self, menu_id):
        self.menu_id = menu_id

    def set_submenu_id(self, submenu_id):
        self.submenu_id = submenu_id

    def set_current_os_flag(self, status):
        if (status == True) or (status == False):
            self.current_os_flag = status

    def set_choosen(self, status):
        if (status == True) or (status == False):
            self.choose_flag = status

    def get_os_name(self):
        return self.os_name

    def get_menu_id(self):
        return self.menu_id

    def get_submenu_id(self):
        return self.submenu_id

    def get_current_os_flag(self):
        return self.current_os_flag

    def get_choosen(self):
        return self.choose_flag

global intern_dbus_name
global intern_dbus_path
global intern_dbus_iface
intern_dbus_name = "org.cdos.bootmgr.client"
intern_dbus_path = "/org/cdos/bootmgr/client"
intern_dbus_iface =  "org.cdos.bootmgr.client"

class OneInstance(dbus.service.Object):

    def __init__(self):
        self.session_bus = dbus.SessionBus()
        name = dbus.service.BusName(intern_dbus_name, bus=self.session_bus)
        dbus.service.Object.__init__(self, name, intern_dbus_path)

    @dbus.service.method(intern_dbus_iface, in_signature='', out_signature='b')
    def show(self):
        return True

if __name__ == "__main__":
    try:
        dbml = DBusGMainLoop(set_as_default=True)
        dbus.set_default_main_loop(dbml)
        bus = dbus.SessionBus()
        remote_object_2 = bus.get_object(intern_dbus_name, intern_dbus_path )
        iface = dbus.Interface(remote_object_2, intern_dbus_iface)
        iface.show()
        exit(0)
    except Exception, e:
        oneinstance = OneInstance()

    boot_mgr = Boot_Manager_UI("/usr/share/boot-manager/frontend")
    # boot_mgr = Boot_Manager_UI("/home/cdos/src/cdos/boot-manger/frontend")
    boot_mgr.show()

    # hide fastboot 
    boot_mgr.fastboot_label.hide()
    boot_mgr.fastboot_switchbutton.hide()

    Gtk.main()
