#!/usr/bin/python
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import os
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GLib,Gio,Gdk,GObject

class FirmwareManager(object):
    def __init__(self, uidir, boot_manager):
        import gettext as gettext2
        gettext2.bindtextdomain('boot_manager', "/usr/share/locale")
        gettext2.textdomain('boot_manager')
        self._ = gettext2.gettext
        provider = Gtk.CssProvider()
        provider.load_from_data(b'''\
            #firmware_reboot_now {
                font-size: 16px;
                background: #44a6e5;
                color: #ffffff;
            }
            #firmware_reboot_now:hover {
                background: #61b2e9;
            }
            #firmware_reboot_now:active {
                background: #3c95cf;
            }
            #firmware_reboot_later,
            #firmware_cancel {
                font-size: 16px;
                background: #dddddd;
                color: #333333;
            }
            #firmware_reboot_later:hover,
            #firmware_cancel:hover {
                background: #e2e2e2;
            }
            #firmware_reboot_later:active,
            #firmware_cancel:active {
                background: #c7c7c7;
            }
            ''')
        Gtk.StyleContext.add_provider_for_screen(
            Gdk.Screen.get_default(),
            provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

        self.boot_mgr = boot_manager
        self.builder = Gtk.Builder()
        self.builder.add_from_file(uidir + "/firmware_manager.ui")
        self.builder.connect_signals(self)
        self.window = self.builder.get_object("firmware_setup_window")
        self.window.set_title(self._("Reboot to BIOS Setup"))
        self.window.set_name("firmware_setup_window")
        self.window.connect("delete-event", self.close)

        self.firmware_label = self.builder.get_object("firmware_label")
        self.firmware_label.set_name("firmware_label")
        self.firmware_label.set_markup(self._("You need to reboot computer to enter BIOS Setup,"))
        self.firmware_notify_label = self.builder.get_object("firmware_notify_label")
        self.firmware_notify_label.set_name("firmware_notify_label")
        self.firmware_notify_label.set_max_width_chars(10)
        self.firmware_notify_label.set_markup(self._("Tips: you will automatically enter BIOS Setup when you reboot your computer next time if you click button \"reboot later\"."))

        self.firmware_reboot_now = self.builder.get_object("firmware_reboot_now")
        self.firmware_reboot_now.set_name("firmware_reboot_now")
        self.firmware_reboot_now.set_label(self._("Reboot Now"))
        self.firmware_reboot_later = self.builder.get_object("firmware_reboot_later")
        self.firmware_reboot_later.set_name("firmware_reboot_later")
        self.firmware_reboot_later.set_label(self._("Reboot Later"))
        self.firmware_cancel = self.builder.get_object("firmware_cancel")
        self.firmware_cancel.set_name("firmware_cancel")
        self.firmware_cancel.set_label(self._("Cancel"))

        self.window.set_position(Gtk.WindowPosition.CENTER)
        self.destroy_flag = False

    def show(self):
        self.boot_mgr.set_sensitive_for_child_window(False)
        self.window.present()
        self.window.show_all()
        self.window.set_keep_above(True)

    def on_firmware_reboot_now_clicked(self, widget):
        os.system("systemctl reboot --firmware-setup")
        self.do_close()

    def on_firmware_reboot_later_clicked(self, widget):
        self.boot_mgr.setreboottofirmwaresetup()
        self.do_close()

    def close(self,  widget, event):
        self.do_close()

    def do_close(self):
        self.boot_mgr.set_sensitive_for_child_window(True)
        self.window.destroy()
        self.destroy_flag = True
        return True

    def on_firmware_cancel_clicked(self, widget):
        self.do_close()
        return True
