#!/bin/python
#-*- coding: utf-8 -*-

##
## Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
##
## This software is published by the license of CPU-OS License, you can use and
## distibuted this software under this License. See CPU-OS Liense for more detail.
##
## You should have receive a copy of CPU-OS License. If not, please contact us
## by email <support_os@cpu-os.ac.cn>
##
##

import os
import shutil
from lib_result import *

def get_grub_env_from_file():
    grub_env_json = {}
    grub_config_path = '/etc/default/grub'
    grub_config_dir = '/etc/default/grub.d/'

    # get config form '/etc/default/grub'
    if not os.path.isfile( grub_config_path):
        return Result( 'Error: /etc/default/grub is not exist', None)
    get_config_from_file_res = get_config_from_file( grub_config_path)
    if get_config_from_file_res.getValue():
        grub_env_json = get_config_from_file_res.getValue()
    # print grub_env_json
    # get config form '/etc/default/grub.d'
    if os.path.isdir( grub_config_dir):
        for file_name in sorted( os.listdir( grub_config_dir)):
            if not file_name.endswith(".cfg"):
                continue
            get_config_from_file_res = get_config_from_file( grub_config_dir + file_name)
            if get_config_from_file_res.getValue():
                tmp_env_json = get_config_from_file_res.getValue()
                # print tmp_env_json
                for env_name in tmp_env_json:
                    if tmp_env_json[env_name].startswith( "\"$" + env_name):
                        # print tmp_env_json[env_name]
                        # print tmp_env_json[env_name][len(env_name)+ 2: -1]
                        grub_env_json[ env_name] = grub_env_json[ env_name][:-1] + tmp_env_json[env_name][len(env_name)+ 2:]
                    else:
                        grub_env_json[ env_name] = tmp_env_json[env_name]
                # print grub_env_json
    # print grub_env_json
    return grub_env_json

def get_config_from_file(config_file_name):
    config_file_json = {}
    try:
        grub_config_msgs = open(config_file_name,'r').read().split('\n')
        for grub_env in grub_config_msgs:
            if ( grub_env != '') and ( not grub_env.startswith('#')):
                grub_env_name = grub_env.split('=')[0]
                grub_env_msg = grub_env[ len(grub_env_name) + 1:]
                config_file_json[ grub_env_name] = grub_env_msg
        return Result(None, config_file_json)
    except Exception as e:
        return Result('Exception: ' + str(e), None)

def get_default_grub_conf():
    config_json = {}
    config_json["timeout"] = 5
    config_json["quiet"] = False
    config_json["splash"] = False
    config_json["maxcpus"] = 0
    config_json["mem"] = 0
    config_json["more_log"] = False
    config_json["debug"] = False
    config_json["serial_debug"] = False
    config_json["usb_debug"] = False
    return config_json

def get_config_from_env(grub_env):
    config_json = get_default_grub_conf()
    if "GRUB_CMDLINE_LINUX_DEFAULT" in grub_env:
        grub_cmd_linux_default_list = grub_env["GRUB_CMDLINE_LINUX_DEFAULT"][1:-1].split()
        grub_cmd_linux_default_json = {}
        for cmd_line in grub_cmd_linux_default_list:
            if cmd_line == "quiet":
                config_json["quiet"] = True
            if cmd_line == "splash":
                config_json["splash"] = True
            # cpu
            if cmd_line == "nosmp":
                config_json["maxcpus"] = 1
            if cmd_line.startswith("maxcpus="):
                try:
                    config_json["maxcpus"] = int(cmd_line.split('=')[1])
                    if config_json["maxcpus"] == 0:
                        config_json["maxcpus"] = 1
                except Exception as e:
                    config_json["maxcpus"] = -1
            # mem
            if cmd_line.startswith("mem="):
                try:
                    mem_capacity = cmd_line.split('=')[1]
                    if mem_capacity[-1] == "K":
                        config_json["mem"] = int(float(mem_capacity[:-1])/1024)
                    elif mem_capacity[-1] == "M":
                        config_json["mem"] = int(mem_capacity[:-1])
                    elif mem_capacity[-1] == "G":
                        config_json["mem"] = int(float(mem_capacity[:-1])*1024)
                    else:
                        config_json["mem"] = 0
                except Exception as e:
                    config_json["mem"] = 0
            # loglevel
            if cmd_line.startswith("loglevel="):
                try:
                    if int(cmd_line.split("=")[1]) == 7:
                        config_json["more_log"] = True
                    else:
                        config_json["more_log"] = False
                except Exception as e:
                    config_json["more_log"] = False
            # Serial Debug
            if cmd_line.startswith("console=ttyS0"):
                config_json["debug"] = True
                config_json["serial_debug"] = True
                config_json["serial_port"] = "ttyS0"
                try:
                    if len(cmd_line.split(",")) > 1:
                        if cmd_line.split(",")[1].endswith("n8"):
                            config_json["serial_baud_rate"] = int(cmd_line.split(",")[1][:-2])
                        else:
                            config_json["serial_baud_rate"] = int(cmd_line.split(",")[1])
                    else:
                        config_json["serial_baud_rate"] = 9600
                except Exception as e:
                    config_json["serial_baud_rate"] = 9600
            # USB Debug
            if cmd_line.startswith("console=ttyUSB0"):
                config_json["debug"] = True
                config_json["usb_debug"] = True
                config_json["usb_port"] = "ttyUSB0"
                try:
                    if len(cmd_line.split(",")) > 1:
                        if cmd_line.split(",")[1].endswith("n8"):
                            config_json["usb_baud_rate"] = int(cmd_line.split(",")[1][:-2])
                        else:
                            config_json["usb_baud_rate"] = int(cmd_line.split(",")[1])
                    else:
                        config_json["usb_baud_rate"] = 9600
                except Exception as e:
                    config_json["usb_baud_rate"] = 9600

    if "GRUB_TIMEOUT" in grub_env:
        config_json["timeout"] = grub_env["GRUB_TIMEOUT"]
    return config_json

def get_os_info_from_cfg(config_file="/boot/grub/grub.cfg"):
    info_10_linux = ""
    info_10_linux_flag = False
    info_10_linux_begin_flag = False
    info_10_linux_finish_flag = False
    info_30_os_prober = ""
    info_30_os_prober_flag = False
    info_30_os_prober_begin_flag = False
    info_30_os_prober_finish_flag = False
    for line in open(config_file).read().split("\n"):
        if "BEGIN /etc/grub.d/10_linux" in line:
            info_10_linux_flag = True
            continue
        if "END /etc/grub.d/10_linux" in line:
            info_10_linux_flag = False
            info_10_linux_begin_flag = False
            info_10_linux_finish_flag = True
            continue
        if "BEGIN /etc/grub.d/30_os-prober" in line:
            info_30_os_prober_flag = True
            continue
        if "END /etc/grub.d/30_os-prober" in line:
            info_30_os_prober_flag = False
            info_30_os_prober_begin_flag = False
            info_30_os_prober_finish_flag = True
            continue
        if info_10_linux_begin_flag:
            if line.startswith("menuentry") or line.startswith("\t") or line.startswith("}"):
                info_10_linux += line + "\n"
            continue
        if info_10_linux_flag and line.startswith("menuentry"):
            # do sth for 10_linux
            info_10_linux_begin_flag = True
            info_10_linux += line + "\n"
            continue
        if info_30_os_prober_begin_flag:
            if line.startswith("menuentry") or line.startswith("\t") or line.startswith("}"):
                info_30_os_prober += line + "\n"
            continue
        if info_30_os_prober_flag and line.startswith("menuentry"):
            # do sth for 10_linux
            info_30_os_prober_begin_flag = True
            info_30_os_prober += line + "\n"
            continue
        if info_10_linux_finish_flag and info_30_os_prober_finish_flag:
            break
    info = {"10_linux": info_10_linux, "30_os_prober": info_30_os_prober}
    return info

def get_10_linux_info_from_cfg(config_file="/boot/grub/grub.cfg"):
    info_10_linux = ""
    info_10_linux_flag = False
    info_10_linux_begin_flag = False
    info_10_linux_finish_flag = False
    for line in open(config_file).read().split("\n"):
        if "BEGIN /etc/grub.d/10_linux" in line:
            info_10_linux_flag = True
            continue
        if "END /etc/grub.d/10_linux" in line:
            info_10_linux_flag = False
            info_10_linux_begin_flag = False
            info_10_linux_finish_flag = True
            break
        if info_10_linux_begin_flag:
            if line.startswith("menuentry") or line.startswith("\t") or line.startswith("}"):
                info_10_linux += line + "\n"
            continue
        if info_10_linux_flag and line.startswith("menuentry"):
            # do sth for 10_linux
            info_10_linux_begin_flag = True
            info_10_linux += line + "\n"
            continue
    return info_10_linux

def get_30_os_prober_info_from_cfg(config_file="/boot/grub/grub.cfg"):
    info_30_os_prober = ""
    info_30_os_prober_flag = False
    info_30_os_prober_begin_flag = False
    info_30_os_prober_finish_flag = False
    for line in open(config_file).read().split("\n"):
        if "BEGIN /etc/grub.d/30_os-prober" in line:
            info_30_os_prober_flag = True
            continue
        if "END /etc/grub.d/30_os-prober" in line:
            info_30_os_prober_flag = False
            info_30_os_prober_begin_flag = False
            info_30_os_prober_finish_flag = True
            break
        if info_30_os_prober_begin_flag:
            if line.startswith("menuentry") or line.startswith("\t") or line.startswith("}"):
                info_30_os_prober += line + "\n"
            continue
        if info_30_os_prober_flag and line.startswith("menuentry"):
            # do sth for 10_linux
            info_30_os_prober_begin_flag = True
            info_30_os_prober += line + "\n"
            continue
    return info_30_os_prober

def get_10_linux_detail_info_from_cfg(config_file="/boot/grub/grub.cfg"):
    info_10_linux_flag = False
    info_10_linux_begin_flag = False
    info_10_linux_finish_flag = False
    submenu_flag = False
    menu_title = menu_id = kernel = menu_root = submenu_id = None
    detail_info = {}
    detail_info["kernels"] = []
    detail_info["kernel_id"] = []
    for line in open(config_file).read().split("\n"):
        if "BEGIN /etc/grub.d/10_linux" in line:
            info_10_linux_flag = True
            continue
        if "END /etc/grub.d/10_linux" in line:
            info_10_linux_flag = False
            info_10_linux_begin_flag = False
            info_10_linux_finish_flag = True
            break
        if info_10_linux_begin_flag:
            if line.startswith("menuentry"):
                menu_id = line.split("\'")[-2]
                if menu_id.startswith("\'") or menu_id.startswith("\""):
                    menu_id = menu_id[1:]
                if menu_id.endswith("\'") or menu_id.endswith("\""):
                    menu_id = menu_id[:-1]
            elif line.startswith("\tset root=") and (not "root" in detail_info):
                menu_root = line.split("=")[1]
                if menu_root.startswith("\'") or menu_root.startswith("\""):
                    menu_root = menu_root[1:]
                if menu_root.endswith("\'") or menu_root.endswith("\""):
                    menu_root = menu_root[:-1]
                detail_info["root"] = menu_root
            elif line.startswith("\tlinux"):
                kernel = get_kernel_version(line.split()[1].split("/")[-1])
                if not kernel in detail_info["kernels"]:
                    detail_info["kernels"].append(kernel)
                    # detail_info["kernel_id"][kernel] = {"menu_id":menu_id,"submenu_id":submenu_id}
                    detail_info["kernel_id"].append({"kernel":kernel,"menu_id":menu_id,"submenu_id":submenu_id})
            elif line.startswith("submenu"):
                submenu_flag = True
                submenu_id = line.split()[-2]
                if submenu_id.startswith("\'") or submenu_id.startswith("\""):
                    submenu_id = submenu_id[1:]
                if submenu_id.endswith("\'") or submenu_id.endswith("\""):
                    submenu_id = submenu_id[:-1]
            elif line.startswith("\tmenuentry"):
                menu_id = line.split("\'")[-2]
                if menu_id.startswith("\'") or menu_id.startswith("\""):
                    menu_id = menu_id[1:]
                if menu_id.endswith("\'") or menu_id.endswith("\""):
                    menu_id = menu_id[:-1]
            elif line.startswith("\t\tlinux"):
                kernel = get_kernel_version(line.split()[1].split("/")[-1])
                if not kernel in detail_info["kernels"]:
                    detail_info["kernels"].append(kernel)
                    # detail_info["kernel_id"][kernel] = {"menu_id":menu_id,"submenu_id":submenu_id}
                    detail_info["kernel_id"].append({"kernel":kernel,"menu_id":menu_id,"submenu_id":submenu_id})
            elif line == "}":
                submenu_flag = False
                submenu_id = None
            continue
        if info_10_linux_flag and line.startswith("menuentry"):
            info_10_linux_begin_flag = True
            menu_title,menu_id = line.split("\'")[1],line.split("\'")[-2]
            if menu_title.startswith("\'") or menu_title.startswith("\""):
                menu_title = menu_title[1:]
            if menu_title.endswith("\'") or menu_title.endswith("\""):
                menu_title = menu_title[:-1]
            if menu_id.startswith("\'") or menu_id.startswith("\""):
                menu_id = menu_id[1:]
            if menu_id.endswith("\'") or menu_id.endswith("\""):
                menu_id = menu_id[:-1]
            detail_info["title"],detail_info["menu_id"] = menu_title,menu_id
            continue
    return detail_info

def get_10_linux_basic_info_from_cfg(config_file="/boot/grub/grub.cfg"):
    info_10_linux_flag = False
    info_10_linux_begin_flag = False
    info_10_linux_finish_flag = False
    menu_title = menu_id = kernel = menu_root = submenu_id = None
    basic_info = {}
    for line in open(config_file).read().split("\n"):
        if "BEGIN /etc/grub.d/10_linux" in line:
            info_10_linux_flag = True
            continue
        if "END /etc/grub.d/10_linux" in line:
            info_10_linux_flag = False
            info_10_linux_begin_flag = False
            info_10_linux_finish_flag = True
            break
        if info_10_linux_begin_flag:
            if line.startswith("\tset root="):
                menu_root = line.split("=")[1]
                if menu_root.startswith("\'") or menu_root.startswith("\""):
                    menu_root = menu_root[1:]
                if menu_root.endswith("\'") or menu_root.endswith("\""):
                    menu_root = menu_root[:-1]
                basic_info["root"] = menu_root
            elif line.startswith("\tlinux"):
                kernel = get_kernel_version(line.split()[1].split("/")[-1])
                basic_info["kernel"] = kernel
            elif line.startswith("submenu"):
                submenu_id = line.split()[-2]
                if submenu_id.startswith("\'") or submenu_id.startswith("\""):
                    submenu_id = submenu_id[1:]
                if submenu_id.endswith("\'") or submenu_id.endswith("\""):
                    submenu_id = submenu_id[:-1]
                basic_info["submenu_id"] = submenu_id
                break
            continue
        if info_10_linux_flag and line.startswith("menuentry"):
            info_10_linux_begin_flag = True
            menu_title,menu_id = line.split("\'")[1],line.split("\'")[-2]
            if menu_title.startswith("\'") or menu_title.startswith("\""):
                menu_title = menu_title[1:]
            if menu_title.endswith("\'") or menu_title.endswith("\""):
                menu_title = menu_title[:-1]
            if menu_id.startswith("\'") or menu_id.startswith("\""):
                menu_id = menu_id[1:]
            if menu_id.endswith("\'") or menu_id.endswith("\""):
                menu_id = menu_id[:-1]
            basic_info["title"],basic_info["menu_id"] = menu_title,menu_id
            continue
    return basic_info

def get_10_linux_config(file_path="/etc/grub.d/10_linux"):
    ubuntu_recovery_value = vt_handoff_value = init_paths = None
    config_json = {}
    for line in open(file_path).read().split("\n"):
        if line.startswith("ubuntu_recovery="):
            ubuntu_recovery_value = line.split("=")[-1]
        if line.startswith("vt_handoff="):
            vt_handoff_value = line.split("=")[-1]
        if line.startswith("SUPPORTED_INITS="):
            init_paths = line.split("=")[-1][1:-1]
        if (ubuntu_recovery_value is not None) and (vt_handoff_value is not None) and (init_paths is not None):
            break
    if (ubuntu_recovery_value is None) or (ubuntu_recovery_value == "0"):
        config_json["recovery"] = False
    else:
        config_json["recovery"] = True
    if (vt_handoff_value is None) or (vt_handoff_value == "0"):
        config_json["vt_handoff"] = False
    else:
        config_json["vt_handoff"] = True
    config_json["inits"] = {}
    if init_paths is not None:
        for init_path in init_paths.split():
            config_json["inits"][init_path.split(":")[0]] = init_path.split(":")[1]
    return config_json

def update_boot_grub_cfg(grub_env, menu_id=None, cmd_linux_defalut_line="\"\"", timeout=None, boot_grub_cfg_file="/boot/grub/grub.cfg", update_menu=False, update_cmdline=False, update_timeout=False):
    try:
        # get config in 10_linux
        up_cfg = get_10_linux_config()
        # get default cmdline
        if cmd_linux_defalut_line.endswith("\n"):
            cmd_linux_defalut_line = cmd_linux_defalut_line[:-1]
        if cmd_linux_defalut_line.startswith("\"") or cmd_linux_defalut_line.startswith("\'"):
            cmd_linux_defalut_line = cmd_linux_defalut_line[1:]
        if cmd_linux_defalut_line.endswith("\"") or cmd_linux_defalut_line.endswith("\'"):
            cmd_linux_defalut_line = cmd_linux_defalut_line[:-1]
        # get cmdline
        if "GRUB_CMDLINE_LINUX" in grub_env:
            cmd_linux_line = grub_env["GRUB_CMDLINE_LINUX"]
        else:
            cmd_linux_line = ""
        if cmd_linux_line.startswith("\"") or cmd_linux_line.startswith("\'"):
            cmd_linux_line = cmd_linux_line[1:]
        if cmd_linux_line.endswith("\"") or cmd_linux_line.endswith("\'"):
            cmd_linux_line = cmd_linux_line[:-1]
        # get recovery title
        if "GRUB_RECOVERY_TITLE" in grub_env:
            recovery_title = grub_env["GRUB_RECOVERY_TITLE"]
        else:
            recovery_title = "recovery mode"
        # get default init type
        try:
            local_init_type = os.readlink("/sbin/init")
        except Exception as e:
            local_init_type = None

        # get init types not default
        init_types = []
        for init_type in up_cfg["inits"]:
            if (local_init_type is None) or (not up_cfg["inits"][init_type] == local_init_type):
                init_types.append(init_type)
        recovery_args = ""
        if os.path.isfile("/lib/recovery-mode/recovery-menu"):
            recovery_args += "recovery"
        else:
            recovery_args += "single"
        if up_cfg["recovery"]:
            recovery_args += " nomodeset"
        write_data = ""

        this_init_type = ""
        update_menu_flag = update_cmdline_flag = update_timeout_flag = False
        recovery_flag = init_flag = default_flag = False
        for line in open(boot_grub_cfg_file).read().split("\n"):
            if update_menu and ("BEGIN /etc/grub.d/00_header" in line):
                update_menu_flag = True
            elif update_menu and update_menu_flag and ("set default=" in line) and (not "{next_entry}" in line):
                write_data += line.split("=")[0] + "=" + menu_id
                update_menu_flag = False
                continue
            elif update_timeout and ("timeout_style=menu" in line):
                update_timeout_flag = True
            elif update_timeout and update_timeout_flag and ("set timeout=" in line):
                write_data += line.split("=")[0] + "=" + timeout
                continue
            elif update_timeout and ("END /etc/grub.d/00_header" in line):
                # print "end update_timeout_flag"
                update_timeout = update_timeout_flag = False
            elif update_cmdline and ("BEGIN /etc/grub.d/10_linux" in line):
                update_cmdline_flag = True
            elif update_cmdline and ("END /etc/grub.d/10_linux" in line):
                update_cmdline_flag = False
            elif update_cmdline and update_cmdline_flag:
                if line.startswith("menuentry") or line.startswith("\tmenuentry"):
                    menu_title = line.split("\'")[1]
                    if recovery_title in menu_title:
                        recovery_flag = True
                    for tmp_init_type in init_types:
                        if tmp_init_type in menu_title:
                            init_flag = True
                            this_init_type = tmp_init_type
                            break
                elif line.startswith("\tlinux") or line.startswith("\t\tlinux"):
                    basic_line = line.split("ro ")[0]+ "ro "
                    if recovery_flag:
                        basic_line += recovery_args + " " + cmd_linux_line
                        recovery_flag = False
                    elif init_flag:
                        basic_line += cmd_linux_line + " " + cmd_linux_defalut_line
                        if up_cfg["vt_handoff"] and ("splash" in cmd_linux_defalut_line):
                            basic_line += " $vt_handoff"
                        basic_line += " init=" + up_cfg["inits"][this_init_type]
                        init_flag = False
                    else:
                        basic_line += cmd_linux_line + " " + cmd_linux_defalut_line
                        if up_cfg["vt_handoff"] and ("splash" in cmd_linux_defalut_line):
                            basic_line += " $vt_handoff"
                    write_data += basic_line + "\n"
                    continue
            write_data += line + "\n"

        open(boot_grub_cfg_file+"_bk","w").write(write_data)
        shutil.copyfile(boot_grub_cfg_file+"_bk", boot_grub_cfg_file)
        return True
    except Exception as e:
        return False

def get_kernels_from_os(boot_dir="/boot/"):
    kernel_list = []
    if not boot_dir.endswith("/"):
        boot_dir += "/"
    for file_name in os.listdir(boot_dir):
        if os.path.isfile(boot_dir + file_name) and file_name.startswith("vmlinuz-"):
            kernel = get_kernel_version(file_name)
            if kernel not in kernel_list:
                kernel_list.append(kernel)
    return kernel_list

def get_30_os_prober_basic_info_from_cfg(config_file="/boot/grub/grub.cfg"):
    if os.path.exists("/cdrom/casper"):
        return []
    info_30_os_prober_flag = False
    info_30_os_prober_begin_flag = False
    info_30_os_prober_finish_flag = False
    submenu_flag = False
    menu_title = menu_id = kernel = menu_root = None
    basic_infos = []
    roots = []
    basic_info = {}
    # example:
    # basic_infos = [{'menu_id': 'osprober-gnulinux-simple-55b24055-6420-4659-ba1f-b2d6870ca706', 'root': 'hd0,msdos1', 'title': 'Ubuntu 17.04 (17.04) (on /dev/sda1)'}]
    for line in open(config_file).read().split("\n"):
        if "BEGIN /etc/grub.d/30_os-prober" in line:
            info_30_os_prober_flag = True
            continue
        if "END /etc/grub.d/30_os-prober" in line:
            info_30_os_prober_flag = False
            info_30_os_prober_begin_flag = False
            info_30_os_prober_finish_flag = True
            break
        if info_30_os_prober_begin_flag:
            if line.startswith("submenu"):
                submenu_flag = True
            elif line.endswith("}") and submenu_flag == True:
                submenu_flag = False
            elif submenu_flag:
                continue
            elif line.startswith("menuentry") or line.startswith("\tmenuentry"):
                # print menu_title,menu_id
                menu_title,menu_id = line.split("\'")[1],line.split("\'")[-2]
                if menu_title.startswith("\'") or menu_title.startswith("\""):
                    menu_title = menu_title[1:]
                if menu_title.endswith("\'") or menu_title.endswith("\""):
                    menu_title = menu_title[:-1]
                if menu_id.startswith("\'") or menu_id.startswith("\""):
                    menu_id = menu_id[1:]
                if menu_id.endswith("\'") or menu_id.endswith("\""):
                    menu_id = menu_id[:-1]
                basic_info["title"],basic_info["menu_id"] = menu_title,menu_id
            #elif (line.startswith("\tset root=") or line.startswith("\t\tset root=")):
            #    menu_root = line.split("=")[1]
            #    if menu_root.startswith("\'") or menu_root.startswith("\""):
            #        menu_root = menu_root[1:]
            #    if menu_root.endswith("\'") or menu_root.endswith("\""):
            #        menu_root = menu_root[:-1]
            elif (line.startswith("  search --no-floppy --fs-uuid --set=root") \
                or line.startswith("\t  search --no-floppy --fs-uuid --set=root") \
                or line.startswith("\t\t  search --no-floppy --fs-uuid --set=root")):
                if menu_title.startswith("方德一键还原"):
                        return []
                s = line.split(" ")
                menu_root = s[len(s) - 1]
                if menu_root in roots:
                    continue
                roots.append(menu_root)
                basic_info["root"] = menu_root
                basic_infos.append(basic_info)
                basic_info = {}
                # print basic_infos
            continue
        if info_30_os_prober_flag and (line.startswith("menuentry") or line.startswith("\tmenuentry")):
            info_30_os_prober_begin_flag = True
            menu_title,menu_id = line.split("\'")[1],line.split("\'")[-2]
            if menu_title.startswith("\'") or menu_title.startswith("\""):
                menu_title = menu_title[1:]
            if menu_title.endswith("\'") or menu_title.endswith("\""):
                menu_title = menu_title[:-1]
            if menu_id.startswith("\'") or menu_id.startswith("\""):
                menu_id = menu_id[1:]
            if menu_id.endswith("\'") or menu_id.endswith("\""):
                menu_id = menu_id[:-1]
            basic_info["title"],basic_info["menu_id"] = menu_title,menu_id
            # print menu_title,menu_id
            continue
    return basic_infos

def get_kernel_version(kernel_name):
    if kernel_name.endswith(".efi.signed"):
        return kernel_name[8:-11]
    if kernel_name.endswith(".efi"):
        return kernel_name[8:-4]
    return kernel_name[8:]


# debug
# print get_grub_env_from_file()
# print get_config_from_env( get_grub_env_from_file())

# get_os_info_from_cfg()
# get_os_info_from_cfg("/home/z/grub.cfg")
# print get_10_linux_info_from_cfg("/home/z/grub.cfg")
# print get_10_linux_info_from_cfg()
# print get_30_os_prober_info_from_cfg("/home/z/grub.cfg")
# print get_30_os_prober_info_from_cfg()

# info = get_10_linux_detail_info_from_cfg("/home/z/grub_cfg/cdos_2_grub.cfg")
# print info
# print info["kernels"]
# for kernel in info["kernel_id"]:
#     print kernel
# print get_10_linux_detail_info_from_cfg("/home/z/grub.cfg_2")
# print get_10_linux_basic_info_from_cfg()
# print get_30_os_prober_basic_info_from_cfg("/home/z/grub_cfg/cdos_2_grub.cfg")
# print get_30_os_prober_basic_info_from_cfg()
# print get_kernels_from_os()

# print get_10_linux_basic_info_from_cfg("/home/z/grub_cfg/cdos_2_grub.cfg")
